scriptDir = 'scripts'
source(file.path(scriptDir, 'setup.R'))

assayMetadata = fread(file.path(paramDir, 'assay_metadata_clinchem.csv'))
ingredMetadata = fread(file.path(paramDir, 'ingred_metadata_clinchem.csv'))

dStatsFp = fread(file.path(resultDir, 'stats_clinchem_falsepos.csv'))
dStatsP = fread(file.path(resultDir, 'stats_clinchem_pos.csv'))

########################################

dStats = rbind(dStatsFp[, .(screen_target, screen_id, ingred_name, or, method = 'falsepos')],
               dStatsP[, .(screen_target, screen_id, ingred_name, or, method = 'pos')])

dStats[, screen_target := str_to_sentence(screen_target)]

dStats = merge(dStats, ingredMetadata, by = c('screen_id', 'ingred_name'),
               all.x = TRUE)

dStats[is.na(ingred_category), ingred_category := 'none of the above']
dStats[, ingred_category :=
         factor(ingred_category, c('assay target', 'known interferent',
                                   'new interferent (intentional)',
                                   'new interferent (incidental)',
                                   'none of the above'))]

dStatsCast = dcast(dStats, screen_target + ingred_name + ingred_category ~ method,
                   value.var = 'or')
setorder(dStatsCast, screen_target, -ingred_category)

########################################

orDummy = 2^(-2)
dStatsCast[is.na(falsepos), falsepos := orDummy]

xLimBreaks = seq(log2(orDummy), 10, 2)
xLimLabels = as.character(xLimBreaks)
xLimLabels[1] = 'NA'

yLimBreaks = seq(log2(orDummy), 10, 2)

dStatsCast[, screen_label := paste(screen_target, 'assay')]
dStatsCast[screen_target == 'Tricyclic antidepressants',
           screen_label := 'TCAs assay']

pOr = ggplot(dStatsCast[screen_target != 'Cocaine metabolite']) +
  facet_wrap(~ screen_label, ncol = 3, scales = 'free') +
  geom_hline(yintercept = 0, color = 'gray') +
  geom_vline(xintercept = 0, color = 'gray') +
  geom_point(aes(x = log2(falsepos), y = log2(pos), fill = ingred_category,
                 color = ingred_category, shape = ingred_category,
                 alpha = ingred_category, size = ingred_category), stroke = 0) +
  labs(x = expression('log'[2]*'(odds ratio for false positives)'),
       y = expression('log'[2]*'(odds ratio for positives)'),
       fill = 'Medication category', color = 'Medication category',
       shape = 'Medication category', alpha = 'Medication category',
       size = 'Medication category') +
  scale_x_continuous(breaks = xLimBreaks, labels = xLimLabels) +
  scale_y_continuous(breaks = yLimBreaks) +
  scale_fill_manual(values = c('#e41a1c', '#377eb8', '#4daf4a', '#984ea3', 'black')) +
  scale_color_manual(values = c('#e41a1c', '#377eb8', '#4daf4a', '#984ea3', 'black')) +
  scale_shape_manual(values = c(22:25, 21),
                     guide = guide_legend(override.aes = list(size = 2.5))) +
  scale_alpha_manual(values = c(rep(0.6, 4), 0.1)) +
  scale_size_manual(values = c(rep(1.8, 4), 1.3), guide = FALSE)

########################################

dStatsCast1 = dStatsCast[ingred_category != 'assay target']
dStatsCast1[, `:=`(rank_falsepos = frank(-falsepos),
                   rank_pos = frank(-pos)),
            by = screen_target]
dStatsCast2 = dStatsCast1[ingred_category != 'none of the above']

pRank = ggplot(dStatsCast2) +
  geom_abline(slope = 1, intercept = 0, color = 'lightgray') +
  geom_point(aes(x = rank_falsepos, y = rank_pos,
                 shape = ingred_category, fill = ingred_category),
             size = 2.25, stroke = 0, alpha = 0.6,
             position = position_jitter(width = 0.05, height = 0.05)) +
  scale_x_log10(limits = c(0.9, 1000)) +
  scale_y_log10(limits = c(0.9, 1000)) +
  scale_fill_manual(values = c('#377eb8', '#4daf4a', '#984ea3')) +
  scale_shape_manual(values = c(23:25)) +
  labs(x = 'Rank for false positives', y = 'Rank for positives',
       fill = 'Medication category', shape = 'Medication category') +
  theme(legend.position = 'none')

########################################

d1 = melt(dStatsCast2, id.vars = c('screen_target', 'ingred_name'),
          measure.vars = c('rank_falsepos', 'rank_pos'),
          variable.name = 'method', value.name = 'rank')
d1[, method := factor(method, labels = c('False positives', 'Positives'))]

d2 = d1[, .(top5 = sum(rank <= 5),
            top10 = sum(rank <= 10),
            top25 = sum(rank <= 25)),
        by = method]

d3 = melt(d2, id.vars = 'method', variable.name = 'k',
          value.name = 'top', variable.factor = FALSE)
d3[, k := factor(as.numeric(str_sub(k, 4)))]

pK = ggplot(d3) +
  geom_col(aes(x = k, y = top, fill = method),
           position = position_dodge(width = 0.8), width = 0.7) +
  scale_fill_brewer(palette = 'Paired') +
  labs(x = 'Top k medications for each assay',
       y = 'Number of interferents\nin the top k',
       fill = 'Method')

########################################

ht = 0.7
wd1 = 0.37
wd2 = 0.55

p = ggdraw() +
  draw_plot(pOr, y = 1 - ht, height = ht) +
  draw_plot(pRank, width = wd1, height = 1 - ht) +
  draw_plot(pK, x = wd1 + 0.02, width = wd2, height = 1 - ht) +
  draw_plot_label(c('A', 'B', 'C'), x = c(0, 0, wd1 + 0.02),
                  y = c(1, 1 - ht, 1 - ht), size = 13)

ggsave(file.path(resultDir, 'stats_clinchem.pdf'), plot = p,
       width = 7.25, height = 8.25, device = cairo_pdf) # unicode requires cairo_pdf
