library('ashr')
library('cba')
library('cowplot')
library('data.table')
library('doParallel')
library('ggplot2')
library('ggstance')
library('lemon')
library('logistf')
library('pheatmap')
library('RODBC')
library('scales')
library('speedglm')
library('stringr')

theme_set(theme_classic() +
            theme(axis.text = element_text(color = 'black'),
                  legend.margin = margin(t = 0, r = 0, b = 0, l = 0, unit = 'cm')))

set.seed(2)
options(datatable.na.strings = '')
registerDoParallel()
paramDir = 'params'
dataDir = 'data'
resultDir = 'results'


sqlDrop = function(con, tableName) {
  sqlQuery(con, sprintf('drop table %s if exists', tableName))}


sqlDropSave = function(con, tableName, dt) {
  # prevent netezza from choking on lowercase column names and table names
  d = copy(dt)
  setnames(d, str_to_upper(colnames(d)))
  sqlDrop(con, tableName)
  sqlSave(con, d, rownames = FALSE, safer = FALSE,
          tablename = str_to_upper(tableName))}


sqlDropQuery = function(con, tableName, q) {
  sqlDrop(con, tableName)
  sqlQuery(con, q)}


sqlFetchDt = function(con, tableName) {
  d = sqlFetch(con, str_to_upper(tableName), stringsAsFactors = FALSE, as.is = TRUE)
  setDT(d)
  setnames(d, str_to_lower(colnames(d)))
  setorder(d)
  return(d)}


getModelInput = function(dScreen, dScreenDrugExp, assayName, ingredNames,
                         respName = 'value_num', assayIdName = 'assay_target') {
  cols = c('person_id', 'measurement_datetime', respName)
  d1 = dScreenDrugExp[get(assayIdName) == assayName & ingred_name %in% ingredNames]
  d2 = dcast(d1, person_id + measurement_datetime ~ ingred_name, fun.aggregate = length)
  d3 = merge(dScreen[get(assayIdName) == assayName, ..cols], d2, by = cols[1:2], all.x = TRUE)
  for (ingredNow in ingredNames) {
    d3[is.na(get(ingredNow)), (ingredNow) := 0]}
  return(d3[])}


getLogisticStats = function(dInput, ingredNames, respName = 'value_num',
                            ingredIdxKeep = 1L, firth = TRUE, ...) {
  formStr = sprintf('%s ~ `%s`', respName, paste(ingredNames, collapse = '` + `'))

  if (isTRUE(firth)) {
    fit = logistf(formula(formStr), data = dInput, ...)
    s = data.table(coef = fit$coefficients,
                   se = sqrt(diag(fit$var)),
                   pval = fit$prob,
                   confint_lower = fit$ci.lower,
                   confint_upper = fit$ci.upper)
    s = s[1L + ingredIdxKeep]

  } else {
    fit = speedglm(formula(formStr), family = binomial(), data = dInput, ...)
    # avoid speedglm converting stuff to factors
    s = data.table(coef(fit)[1L + ingredIdxKeep],
                   coef(summary(fit))[1L + ingredIdxKeep, 2:3, drop = FALSE])
    setnames(s, c('coef', 'se', 'z'))
    s[, pval := 2 * pnorm(-abs(z))]
    s[, z := NULL]
    ci = confint(fit)[1L + ingredIdxKeep, , drop = FALSE]
    s[, `:=`(confint_lower = ci[, 1L], confint_upper = ci[, 2L])]}
  return(s[])}


getLinearStats = function(dInput, ingredNames, respName = 'value_num',
                          ingredIdxKeep = 1L) {
  formStr = sprintf('%s ~ `%s`', respName, paste(ingredNames, collapse = '` + `'))
  fit = lm(formula(formStr), data = dInput)
  s = as.data.table(coef(summary(fit))[1L + ingredIdxKeep, -3L, drop = FALSE])
  setnames(s, c('coef', 'se', 'pval'))
  ci = confint(fit)[1L + ingredIdxKeep, , drop = FALSE]
  s[, `:=`(confint_lower = ci[, 1L], confint_upper = ci[, 2L])]
  return(s[])}


getAshStats = function(dStats, mixcompdist = 'halfuniform', ...) {
  ashObj = ash(dStats$coef, dStats$se, mixcompdist = mixcompdist, ...)
  dAsh = setDT(ashObj$result)[, !c('betahat', 'sebetahat')]
  ci = ashci(ashObj)
  dAsh[, `:=`(credint_lower = ci[, 1L], credint_upper = ci[, 2L])]
  return(dAsh[])}


getOrderedMat = function(mat) {
  if (nrow(mat) < 3) {
    return(mat)}
  dd = dist(mat)
  co = order.optimal(dd, hclust(dd)$merge)
  mat = mat[co$order, ]
  mat = mat[, co$order]
  return(mat)}


getCoexpFreq = function(dWide, ingredNames, cluster = TRUE) {
  # fraction of exposures to row ingred, in which person also exposed to col ingred
  d = copy(dWide)
  d[, idx := 1:.N]
  d = melt(d, id.vars = 'idx', measure.vars = ingredNames, variable.name = 'ingred')

  d1 = merge(d[, .(idx, ingred1 = ingred, val1 = value)],
             d[, .(idx, ingred2 = ingred, val2 = value)],
             by = 'idx', allow.cartesian = TRUE)
  d2 = d1[, .(v = sum(val1 * val2) / sum(val1)), by = .(ingred1, ingred2)]
  d3 = dcast(d2, ingred1 ~ ingred2, value.var = 'v')

  mat = as.matrix(d3[, 2:ncol(d3)])
  rownames(mat) = colnames(mat)

  if (isTRUE(cluster)) {
    mat = getOrderedMat(mat)}
  diag(mat) = NA
  return(mat)}


plotCoexpFreq = function(coexpMat, dStats, main, filepath, palette = 'Greys',
                         maxChars = 20, width = 4.5, height = 3) {

  drugAnnoColors = list(`OR` = scales::brewer_pal('seq', palette)(5))
  drugAnnoPre = dStats[, .(ingred_name, `OR` = round(exp(PosteriorMean), 2))]

  drugAnno = data.frame(ingred_name = rownames(coexpMat), stringsAsFactors = FALSE)
  drugAnno = merge(drugAnno, drugAnnoPre, by = 'ingred_name', sort = FALSE)
  rownames(drugAnno) = rownames(coexpMat)
  drugAnno$ingred_name = NULL

  drugsShort = ifelse(nchar(rownames(coexpMat)) <= maxChars, rownames(coexpMat),
                      paste0(substr(rownames(coexpMat), 1, maxChars - 2), '...'))

  pheatmap(coexpMat, color = colorRampPalette(RColorBrewer::brewer.pal(7, 'YlGnBu'))(100),
           cluster_rows = FALSE, cluster_cols = FALSE,
           annotation_row = drugAnno, annotation_col = drugAnno,
           annotation_colors = drugAnnoColors,
           labels_row = drugsShort, labels_col = drugsShort, main = main,
           filename = filepath, width = width, height = height, silent = TRUE)}


plotSpiking = function(d, xBreaks, yBreaks, dInterp, xText, yText) {
  dInterp = merge(dInterp, unique(d[, .(test_compound, test_compound_fac)]),
                  by = 'test_compound')
  dInterp[, label := sprintf('%.1f \u03bcg/mL', test_conc_cutoff)]
  dInterp[is.na(test_conc_cutoff), label := 'NA']

  # https://github.com/tidyverse/ggplot2/issues/980
  mysqrt_trans = trans_new('mysqrt', transform = base::sqrt,
                           inverse = function(x) ifelse(x < 0, 0, x^2),
                           domain = c(0, Inf))

  p = ggplot(d) +
    facet_wrap(vars(test_compound_fac)) +
    geom_text(aes(x = xText, y = yText, label = label), size = 3, hjust = 'right',
              data = dInterp) +
    geom_hline(yintercept = cutoffAcet, color = 'gray', linetype = 'dashed') +
    geom_line(aes(x = test_conc, y = target_conc)) +
    geom_point(aes(x = test_conc, y = target_conc), shape = 16) +
    labs(x = 'Test conc. (\u03bcg/mL)') +
    scale_x_continuous(trans = mysqrt_trans, limits = c(0, NA), breaks = xBreaks) +
    scale_y_continuous(trans = mysqrt_trans, limits = c(0, NA), breaks = yBreaks) +
    theme(axis.title.y = element_blank())
  return(p)}
